<?php

namespace Drupal\cl_block\Plugin\Filter;

use Drupal\Core\Annotation\Translation;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Template\TwigEnvironment;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a filter that allows text to be rendered using the Twig engine.
 *
 * @Filter(
 *   id = "twig_code",
 *   title = @Translation("Twig input filter"),
 *   description= @Translation("Uses the Twig environment to render the input text."),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_HTML_RESTRICTOR,
 *   settings = {},
 *   weight = -22
 * )
 */
class TwigInputFilter extends FilterBase implements ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\Core\Template\TwigEnvironment
   */
  private TwigEnvironment $twig;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, TwigEnvironment $twig) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->twig = $twig;
  }


  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $twig_environment = $container->get('twig');
    return new static($configuration, $plugin_id, $plugin_definition, $twig_environment);
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {
    $processed_text = (string) $this->twig->renderInline($text, ['langcode' => $langcode]);
    return new FilterProcessResult($processed_text);
  }

  /**
   * {@inheritdoc}
   */
  public function tips($long = FALSE, $context = []) {
    return $this->t('Use the Twig templating engine to render the text. See <a href=":url">@url</a> for more information.', [
      ':url' => 'http://twig.sensiolabs.org/documentation',
      '@url' => 'http://twig.sensiolabs.org/documentation',
    ]);
  }

}
